package guru.kake.ronove

import javafx.application.Platform
import javafx.beans.property.SimpleIntegerProperty
import javafx.event.EventHandler
import javafx.scene.control.*
import javafx.scene.input.KeyEvent
import tornadofx.*

/**
 * The central view of the main window used to shew the table of the translation data
 */
class TranslationPageView : View() {
    private val project = (app as Ronove).project

    init {
        PageJumpMenu.pageView = this
    }

    private lateinit var tableView: TableView<Line>
    private lateinit var pageJumps: PageJumpMenu
    var currentPage = SimpleIntegerProperty(0)
    var pageCount = project.pages.size.toProperty()

    override val root = borderpane {
        tableView = tableview(project.pages[0].lines.asObservable()) {
            isEditable = true

            column("Original", Line::original).makeEditable()
            column("Machine translation", Line::machineTranslation).makeEditable()
            column("Final", Line::finalTranslation).makeEditable()
            column("Notes", Line::notes).makeEditable()

            contextmenu {}
        }
        tableView.selectionModel.isCellSelectionEnabled = true
        tableView.selectionModel.selectedItemProperty().addListener(fun(_, _, newSelection) {
            if (newSelection != null) {

                val row: Int = tableView.selectionModel.selectedCells[0].row
                val col = tableView.columns[tableView.selectionModel.selectedCells[0].column]
                Platform.runLater { tableView.edit(row, col) }
            }
        })
        tableView.onKeyPressed = EventHandler { event: KeyEvent ->
            val pos: TablePosition<*, *>? = tableView.focusModel.focusedCell
            if (pos != null && event.code.isLetterKey) {
                @Suppress("UNCHECKED_CAST")
                tableView.edit(pos.row, pos.tableColumn as TableColumn<Line, *>?)
            }
        }
        center = tableView

        pageJumps = PageJumpMenu()
        bottom = pageJumps.root
    }

    /**
     * Change the page to edit and load the page to the view
     * @param page Page to change to
     */
    fun changePage(page: Int) {
        currentPage.value = page
    }

    /**
     * Loads the currently selected page to the table view
     */
    private fun refresh() {
        tableView.items = project.pages[currentPage.value].lines.asObservable()
    }

    init {
        currentPage.addListener { _ -> refresh(); pageJumps.refresh()}
        pageCount.addListener { _ -> pageJumps.refresh()}
    }
}

/**
 * Fragment of the main window implementing the page switcher at the bottom
 */
class PageJumpMenu : Fragment() {
    companion object {
        lateinit var pageView: TranslationPageView
    }

    /**
     * Refresh the page switcher, updating the selected page and number of pages
     */
    fun refresh() = replaceWith<PageJumpMenu>()

    private fun changePage(page: Int) = pageView.changePage(page)

    override val root = hbox {
        addClass(RonoveStylesheet.vcenter)
        button("<<") {action {changePage(1)}}
        button("<")
        for (i in 1..5) {
            button("$i") {action {changePage(i)}}
        }
        menubutton("...") {
            for (i in 1..pageView.pageCount.get()) {
                item("$i") {action {changePage(i)}}
            }
        }
        button(">")
        button(">>")
        button("+") {
            action {
                //TODO: add page to project
            }
        }
    }
}